@@ -778,7 +778,10 @@ static POA_ConfigDatabase3__epv server3_epv = {
 
 static POA_ConfigDatabase3__vepv poa_server_vepv = { &base_epv, &server_epv, &server2_epv, &server3_epv };
 
-static void gconf_database_really_sync(GConfDatabase* db);
+static void gconf_database_really_sync (GConfDatabase *db);
+static void source_notify_cb           (GConfSource   *source,
+					const gchar   *location,
+					GConfDatabase *db);
 
 GConfDatabase*
 gconf_database_new (GConfSources  *sources)
@@ -810,6 +813,10 @@ gconf_database_new (GConfSources  *sources)
 
   db->sources = sources;
 
+  gconf_sources_set_notify_func (db->sources,
+				 (GConfSourceNotifyFunc) source_notify_cb,
+				 db);
+
   db->last_access = time(NULL);
 
   db->sync_idle = 0;
@@ -1001,7 +1008,59 @@ gconf_database_schedule_sync(GConfDatabase* db)
   else
     {
       /* 1 minute timeout */
-      db->sync_timeout = g_timeout_add(60000, (GSourceFunc)gconf_database_sync_timeout, db);
+      db->sync_timeout = g_timeout_add_seconds(60, (GSourceFunc)gconf_database_sync_timeout, db);
+    }
+}
+
+static void
+source_notify_cb (GConfSource   *source,
+		  const gchar   *location,
+		  GConfDatabase *db)
+{
+  g_return_if_fail (source != NULL);
+  g_return_if_fail (location != NULL);
+  g_return_if_fail (db != NULL);
+
+  if (gconf_sources_is_affected (db->sources, source, location))
+    {
+      GConfValue  *value;
+      ConfigValue *cvalue;
+      GError      *error;
+      gboolean     is_default;
+      gboolean     is_writable;
+
+      error = NULL;
+      is_default = is_writable = FALSE;
+
+      value = gconf_database_query_value (db,
+					  location,
+					  NULL,
+					  TRUE,
+					  NULL,
+					  &is_default,
+					  &is_writable,
+					  &error);
+      if (error != NULL)
+	{
+	  gconf_log (GCL_WARNING,
+		     _("Error obtaining new value for `%s' after change notification from backend `%s': %s"),
+		     location,
+		     source->address,
+		     error->message);
+	  g_error_free (error);
+	  return;
+	}
+
+      cvalue = gconf_corba_value_from_gconf_value (value);
+      gconf_database_notify_listeners (db,
+				       NULL,
+				       location,
+				       cvalue,
+				       is_default,
+				       is_writable,
+				       FALSE);
+      CORBA_free (cvalue);
+      gconf_value_free (value);
     }
 }
 
@@ -1025,6 +1084,8 @@ gconf_database_readd_listener   (GConfDatabase       *db,
   cnxn = gconf_listeners_add (db->listeners, where, l,
                               (GFreeFunc)listener_destroy);
 
+  gconf_sources_add_listener (db->sources, cnxn, where);
+
   if (l->name == NULL)
     l->name = g_strdup_printf ("%u", cnxn);
   
@@ -1055,7 +1116,7 @@ gconf_database_add_listener    (GConfDatabase       *db,
        * app to simply continue.
        */
       gconf_log (GCL_WARNING,
-		 _("Failed to log addition of listener %s (%s);"
+		 _("Failed to log addition of listener %s (%s); "
 		   "will not be able to restore this listener on "
 		   "gconfd restart, resulting in unreliable "
 		   "notification of configuration changes."),
@@ -1108,6 +1169,8 @@ gconf_database_remove_listener (GConfDatabase       *db,
       g_error_free (err);
     }
   
+  gconf_sources_remove_listener (db->sources, cnxn);
+
   /* calls destroy notify */
   gconf_listeners_remove (db->listeners, cnxn);
 }
@@ -1160,10 +1223,12 @@ notify_listeners_cb(GConfListeners* listeners,
 
 void
 gconf_database_notify_listeners (GConfDatabase       *db,
+				 GConfSources        *modified_sources,
                                  const gchar         *key,
                                  const ConfigValue   *value,
                                  gboolean             is_default,
-                                 gboolean             is_writable)
+                                 gboolean             is_writable,
+				 gboolean             notify_others)
 {
   ListenerNotifyClosure closure;
   GSList* tmp;
@@ -1193,6 +1258,17 @@ gconf_database_notify_listeners (GConfDatabase       *db,
 
       tmp = g_slist_next(tmp);
     }
+
+  g_slist_free (closure.dead);
+
+  if (modified_sources)
+    {
+      if (notify_others)
+          gconfd_notify_other_listeners (db, modified_sources, key);
+
+      g_list_free (modified_sources->sources);
+      g_free (modified_sources);
+    }
 }
 
 GConfValue*
@@ -1253,6 +1329,7 @@ gconf_database_set   (GConfDatabase      *db,
                       GError        **err)
 {
   GError *error = NULL;
+  GConfSources *modified_sources;
   
   g_assert(db->listeners != NULL);
   g_return_if_fail(err == NULL || *err == NULL);
@@ -1264,10 +1341,12 @@ gconf_database_set   (GConfDatabase      *db,
   gconf_log(GCL_DEBUG, "Received request to set key `%s'", key);
 #endif
   
-  gconf_sources_set_value(db->sources, key, value, &error);
+  gconf_sources_set_value(db->sources, key, value, &modified_sources, &error);
 
   if (error)
     {
+      g_assert (modified_sources == NULL);
+
       gconf_log(GCL_ERR, _("Error setting value for `%s': %s"),
                 key, error->message);
       
@@ -1279,14 +1358,16 @@ gconf_database_set   (GConfDatabase      *db,
     {
       gconf_database_schedule_sync(db);
       
-      gconf_database_notify_listeners(db, key, cvalue,
-                                      /* Can't possibly be the default,
-                                         since we just set it,
-                                         and must be writable since
-                                         setting it succeeded.
-                                      */
-                                      FALSE,
-                                      TRUE);
+      /* Can't possibly be the default, since we just set it,
+       * and must be writable since setting it succeeded.
+       */
+      gconf_database_notify_listeners (db,
+				       modified_sources,
+				       key,
+				       cvalue,
+                                       FALSE,
+				       TRUE,
+				       TRUE);
     }
 }
 
@@ -1298,6 +1379,7 @@ gconf_database_unset (GConfDatabase      *db,
 {
   ConfigValue* val;
   GError* error = NULL;
+  GConfSources *modified_sources = NULL;
   
   g_return_if_fail(err == NULL || *err == NULL);
   
@@ -1307,10 +1389,12 @@ gconf_database_unset (GConfDatabase      *db,
   
   gconf_log(GCL_DEBUG, "Received request to unset key `%s'", key);
 
-  gconf_sources_unset_value(db->sources, key, locale, &error);
+  gconf_sources_unset_value(db->sources, key, locale, &modified_sources, &error);
 
   if (error != NULL)
     {
+      g_assert (modified_sources == NULL);
+
       gconf_log(GCL_ERR, _("Error unsetting `%s': %s"),
                 key, error->message);
 
@@ -1355,8 +1439,13 @@ gconf_database_unset (GConfDatabase      *db,
           
       gconf_database_schedule_sync(db);
 
-      gconf_database_notify_listeners(db, key, val, TRUE, is_writable);
-      
+      gconf_database_notify_listeners(db,
+				      modified_sources,
+				      key,
+				      val,
+				      TRUE,
+				      is_writable,
+				      TRUE);
       CORBA_free(val);
     }
 }
@@ -1391,6 +1480,8 @@ gconf_database_recursive_unset (GConfDatabase      *db,
    */
   if (error != NULL)
     {
+      g_assert (notifies == NULL);
+
       gconf_log (GCL_ERR, _("Error unsetting \"%s\": %s"),
                  key, error->message);
 
@@ -1409,24 +1500,24 @@ gconf_database_recursive_unset (GConfDatabase      *db,
       const gchar* locale_list[] = { NULL, NULL };
       gboolean is_writable = TRUE;
       gboolean is_default = TRUE;
-      char *notify_key = tmp->data;
-      
+      GConfUnsetNotify *notify = tmp->data;
 
       locale_list[0] = locale;
       new_value = gconf_database_query_value (db,
-                                              notify_key,
+                                              notify->key,
                                               locale_list,
                                               TRUE,
                                               NULL,
                                               &is_default,
                                               &is_writable,
                                               &error);
-
       if (error)
-        gconf_log (GCL_ERR, _("Error getting new value for \"%s\": %s"),
-                   notify_key, error->message);
-      g_propagate_error (err, error);
-      error = NULL;
+	{
+	  gconf_log (GCL_ERR, _("Error getting new value for \"%s\": %s"),
+		     notify->key, error->message);
+	  g_propagate_error (err, error);
+	  error = NULL;
+	}
       
       if (new_value != NULL)
         {
@@ -1440,11 +1531,17 @@ gconf_database_recursive_unset (GConfDatabase      *db,
           
       gconf_database_schedule_sync (db);
 
-      gconf_database_notify_listeners (db, notify_key, val,
-                                       is_default, is_writable);
+      gconf_database_notify_listeners (db,
+				       notify->modified_sources,
+				       notify->key,
+				       val,
+                                       is_default,
+				       is_writable,
+				       TRUE);
       
       CORBA_free (val);
-      g_free (notify_key);
+      g_free (notify->key);
+      g_free (notify);
       
       tmp = tmp->next;
     }
@@ -1586,7 +1683,7 @@ gconf_database_sync (GConfDatabase  *db,
   
   db->last_access = time(NULL);
   
-  gconf_log(GCL_DEBUG, "Received suggestion to sync all config data");
+  gconf_log(GCL_DEBUG, "Received suggestion to sync all configuration data");
 
   gconf_database_sync_nowish(db);
 }
@@ -1624,18 +1721,44 @@ gconf_database_clear_cache (GConfDatabase  *db,
   gconf_sources_clear_cache(db->sources);
 }
 
-const gchar*
+const gchar *
 gconf_database_get_persistent_name (GConfDatabase *db)
 {
-  if (db->persistent_name == NULL)
+  GList   *tmp;
+  GString *str = NULL;
+
+  if (db->persistent_name != NULL)
+    return db->persistent_name;
+
+  if (db->sources == NULL || db->sources->sources == NULL)
     {
-      if (db->sources->sources)
-        db->persistent_name =
-          g_strdup (((GConfSource*)db->sources->sources->data)->address);
-      else
-        db->persistent_name = g_strdup ("empty");
+      db->persistent_name = g_strdup ("empty");
+      return db->persistent_name;
     }
 
+  tmp = db->sources->sources;
+  while (tmp != NULL)
+    {
+      GConfSource *source = tmp->data;
+
+      if (str == NULL)
+	{
+	  str = g_string_new (source->address);
+	}
+      else
+        {
+          g_string_append_c (str, GCONF_DATABASE_LIST_DELIM);
+          g_string_append (str, source->address);
+        }
+
+      tmp = tmp->next;
+    }
+
+  g_assert (str != NULL);
+
+  db->persistent_name = str->str;
+  g_string_free (str, FALSE);
+
   return db->persistent_name;
 }
 
