@@ -49,15 +49,14 @@
 #include <fcntl.h>
 #include <errno.h>
 #include <ctype.h>
-#ifdef HAVE_SYSLOG_H
-#include <syslog.h>
-#endif
 #include <time.h>
 #ifdef HAVE_SYS_WAIT_H
 #include <sys/wait.h>
 #endif
 #include <locale.h>
 
+#include <dbus/dbus-glib-lowlevel.h>
+
 /* This makes hash table safer when debugging */
 #ifndef GCONF_ENABLE_DEBUG
 #define safe_g_hash_table_insert g_hash_table_insert
@@ -351,7 +350,7 @@ gconf_server_load_sources(void)
       /* Try using the default address xml:readwrite:$(HOME)/.gconf */
       addresses = g_slist_append(addresses, g_strconcat("xml:readwrite:", home, "/.gconf", NULL));
 
-      gconf_log(GCL_DEBUG, _("No configuration files found, trying to use the default config source `%s'"), (char *)addresses->data);
+      gconf_log(GCL_DEBUG, _("No configuration files found. Trying to use the default configuration source `%s'"), (char *)addresses->data);
     }
   
   if (addresses == NULL)
@@ -359,7 +358,7 @@ gconf_server_load_sources(void)
       /* We want to stay alive but do nothing, because otherwise every
          request would result in another failed gconfd being spawned.  
       */
-      gconf_log(GCL_ERR, _("No configuration sources in the source path, configuration won't be saved; edit %s%s"), GCONF_CONFDIR, "/path");
+      gconf_log(GCL_ERR, _("No configuration sources in the source path. Configuration won't be saved; edit %s%s"), GCONF_CONFDIR, "/path");
       /* don't request error since there aren't any addresses */
       sources = gconf_sources_new_from_addresses(NULL, NULL);
 
@@ -372,7 +371,7 @@ gconf_server_load_sources(void)
 
       if (error != NULL)
         {
-          gconf_log(GCL_ERR, _("Error loading some config sources: %s"),
+          gconf_log(GCL_ERR, _("Error loading some configuration sources: %s"),
                     error->message);
 
           g_error_free(error);
@@ -384,7 +383,7 @@ gconf_server_load_sources(void)
       g_assert(sources != NULL);
 
       if (sources->sources == NULL)
-        gconf_log(GCL_ERR, _("No config source addresses successfully resolved, can't load or store config data"));
+        gconf_log(GCL_ERR, _("No configuration source addresses successfully resolved. Can't load or store configuration data"));
     
       tmp = sources->sources;
 
@@ -401,7 +400,7 @@ gconf_server_load_sources(void)
 
       /* In this case, some sources may still return TRUE from their writable() function */
       if (!have_writable)
-        gconf_log(GCL_WARNING, _("No writable config sources successfully resolved, may not be able to save some configuration changes"));
+        gconf_log(GCL_WARNING, _("No writable configuration sources successfully resolved. May be unable to save some configuration changes"));
 
         
       /* Install the sources as the default database */
@@ -409,6 +408,10 @@ gconf_server_load_sources(void)
     }
 }
 
+/*
+ * Signal handlers should not log debug messages as this code is non-reentrant.
+ * Please avoid calling gconf_log in this function.
+ */
 static void
 signal_handler (int signo)
 {
@@ -429,15 +432,9 @@ signal_handler (int signo)
   case SIGILL:
     enter_shutdown ();
 #ifndef G_OS_WIN32
-    gconf_log (GCL_ERR,
-               _("Received signal %d, dumping core. Please report a GConf bug."),
-               signo);
     if (g_getenv ("DISPLAY"))
       gconf_handle_segv (signo);
 #else
-    gconf_log (GCL_ERR,
-               _("Received signal %d. Please report a GConf bug."),
-               signo);
     gconf_handle_segv (signo);
 #endif
     abort ();
@@ -455,11 +452,6 @@ signal_handler (int signo)
     /* let the fatal signals interrupt us */
     --in_fatal;
     
-    gconf_log (GCL_ERR,
-               _("Received signal %d, shutting down abnormally. Please file a GConf bug report."),
-               signo);
-
-
     if (gconf_main_is_running ())
       gconf_main_quit ();
     
@@ -471,9 +463,6 @@ signal_handler (int signo)
     /* let the fatal signals interrupt us */
     --in_fatal;
     
-    gconf_log (GCL_INFO,
-               _("Received signal %d, shutting down cleanly"), signo);
-
     if (gconf_main_is_running ())
       gconf_main_quit ();
     break;
@@ -510,92 +499,122 @@ gconf_get_poa (void)
   return the_poa;
 }
 
-static void
-log_handler (const gchar   *log_domain,
-             GLogLevelFlags log_level,
-             const gchar   *message,
-             gpointer       user_data)
+static const char *
+get_introspection_xml (void)
 {
-  GConfLogPriority pri = GCL_WARNING;
-  
-  switch (log_level & G_LOG_LEVEL_MASK)
-    {
-    case G_LOG_LEVEL_ERROR:
-    case G_LOG_LEVEL_CRITICAL:
-      pri = GCL_ERR;
-      break;
-
-    case G_LOG_LEVEL_WARNING:
-      pri = GCL_WARNING;
-      break;
-
-    case G_LOG_LEVEL_MESSAGE:
-    case G_LOG_LEVEL_INFO:
-      pri = GCL_INFO;
-      break;
-
-    case G_LOG_LEVEL_DEBUG:
-      pri = GCL_DEBUG;
-      break;
-
-    default:
-      break;
-    }
-
-  gconf_log (pri, "%s", message);
+  return "<!DOCTYPE node PUBLIC \"-//freedesktop//DTD D-BUS Object Introspection 1.0//EN\"\n"
+         "\"http://www.freedesktop.org/standards/dbus/1.0/introspect.dtd\">\n"
+         "<node>\n"
+         "  <interface name=\"org.freedesktop.DBus.Introspectable\">\n"
+         "    <method name=\"Introspect\">\n"
+         "      <arg name=\"introspection_xml\" direction=\"out\" type=\"s\"/>\n"
+         "    </method>\n"
+         "  </interface>\n"
+         "  <interface name=\"org.gnome.GConf\">\n"
+         "    <method name=\"GetIOR\">\n"
+         "      <arg name=\"ior\" direction=\"out\" type=\"s\"/>\n"
+         "    </method>\n"
+         "  </interface>\n"
+         "</node>\n";
 }
 
-/* From ORBit2 */
-/* There is a DOS attack if another user creates
- * the given directory and keeps us from creating
- * it
- */
-static gboolean
-test_safe_tmp_dir (const char *dirname)
+static DBusHandlerResult
+bus_message_handler (DBusConnection *connection,
+                     DBusMessage    *message,
+                     GMainLoop      *loop)
 {
-#ifndef G_OS_WIN32
-  struct stat statbuf;
-  int fd;
+  DBusMessage *reply;
 
-  fd = open (dirname, O_RDONLY);  
-  if (fd < 0)
-    {
-      gconf_log (GCL_WARNING, _("Failed to open %s: %s"),
-                 dirname, g_strerror (errno));
-      return FALSE;
-    }
-  
-  if (fstat (fd, &statbuf) != 0)
-    {
-      gconf_log (GCL_WARNING, _("Failed to stat %s: %s"),
-                 dirname, g_strerror (errno));
-      close (fd);
-      return FALSE;
-    }
-  close (fd);
+  reply = NULL;
 
-  if (statbuf.st_uid != getuid ())
+  if (dbus_message_is_signal (message,
+                              DBUS_INTERFACE_LOCAL,
+                              "Disconnected"))
     {
-      gconf_log (GCL_WARNING, _("Owner of %s is not the current user"),
-                 dirname);
-      return FALSE;
+      gconf_main_quit ();
+      return DBUS_HANDLER_RESULT_HANDLED;
+    }
+  else if (dbus_message_is_method_call (message,
+                                        "org.freedesktop.DBus.Introspectable",
+                                        "Introspect"))
+    {
+      const char *introspection_xml;
+
+      introspection_xml = get_introspection_xml ();
+
+      reply = dbus_message_new_method_return (message);
+      dbus_message_append_args (reply, DBUS_TYPE_STRING, &introspection_xml,
+                                DBUS_TYPE_INVALID);
+
+    }
+  else if (dbus_message_is_method_call (message,
+                                        "org.gnome.GConf",
+                                        "GetIOR"))
+    {
+      const char *ior;
+
+      ior = gconf_get_daemon_ior ();
+
+      reply = dbus_message_new_method_return (message);
+      dbus_message_append_args (reply, DBUS_TYPE_STRING, &ior, DBUS_TYPE_INVALID);
     }
 
-  if ((statbuf.st_mode & (S_IRWXG|S_IRWXO)) ||
-      !S_ISDIR (statbuf.st_mode))
+  if (reply != NULL)
     {
-      gconf_log (GCL_WARNING, _("Bad permissions %lo on directory %s"),
-                 (unsigned long) statbuf.st_mode & 07777, dirname);
-      return FALSE;
+      dbus_connection_send (connection, reply, NULL);
+      dbus_message_unref (reply);
+      return DBUS_HANDLER_RESULT_HANDLED;
     }
-#else
-  /* FIXME: We can't get any useful information about the actual
-   * protection for the directory using stat(). We must use the Win32
-   * API to check the owner and permissions (ACL). Later.
-   */
-#endif
-  
-  return TRUE;
+
+  return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+}
+
+static DBusConnection *
+get_on_d_bus (void)
+{
+  DBusConnection *connection;
+  DBusError bus_error;
+  int result;
+
+  dbus_error_init (&bus_error);
+  connection = dbus_bus_get (DBUS_BUS_SESSION, &bus_error);
+
+  if (dbus_error_is_set (&bus_error))
+    {
+      gconf_log (GCL_ERR, _("Could not connect to session bus: %s"), bus_error.message);
+      dbus_error_free (&bus_error);
+      return NULL;
+    }
+
+  dbus_connection_setup_with_g_main (connection, NULL);
+
+  if (!dbus_connection_add_filter (connection, (DBusHandleMessageFunction)
+                                  bus_message_handler, NULL, NULL))
+    {
+      dbus_connection_unref (connection);
+      return NULL;
+    }
+
+  dbus_connection_set_exit_on_disconnect (connection, FALSE);
+
+  result = dbus_bus_request_name (connection, "org.gnome.GConf",
+                                  0, &bus_error);
+
+  if (dbus_error_is_set (&bus_error))
+    {
+      gconf_log (GCL_WARNING,
+                 _("Failed to get bus name for daemon, exiting: %s"),
+                 bus_error.message);
+      dbus_error_free (&bus_error);
+    }
+
+  if (result != DBUS_REQUEST_NAME_REPLY_PRIMARY_OWNER)
+    {
+      dbus_connection_unref (connection);
+      return NULL;
+    }
+
+  return connection;
 }
 
 int 
@@ -604,18 +623,16 @@ main(int argc, char** argv)
 #ifdef HAVE_SIGACTION
   struct sigaction act;
   sigset_t empty_mask;
+  sigset_t full_mask;
 #endif
   CORBA_Environment ev;
   CORBA_ORB orb;
-  gchar* logname;
-  const gchar* username;
   gchar* ior;
   int exit_code = 0;
   GError *err;
-  char *lock_dir;
-  char *gconfd_dir;
   int dev_null_fd;
   int write_byte_fd;
+  DBusConnection *connection;
 
   _gconf_init_i18n ();
   setlocale (LC_ALL, "");
@@ -656,27 +673,7 @@ main(int argc, char** argv)
   
   gconf_set_daemon_mode(TRUE);
   
-  /* Logs */
-  username = g_get_user_name();
-  logname = g_strdup_printf("gconfd (%s-%u)", username, (guint)getpid());
-
-#ifdef HAVE_SYSLOG_H
-  openlog (logname, LOG_NDELAY, LOG_USER);
-#endif
-
-  g_log_set_handler (NULL, G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION,
-                     log_handler, NULL);
-
-  g_log_set_handler ("GLib", G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION,
-                     log_handler, NULL);
-
-  g_log_set_handler ("GLib-GObject", G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION,
-                     log_handler, NULL);
-  
-  /* openlog() does not copy logname - what total brokenness.
-     So we free it at the end of main() */
-  
-  gconf_log (GCL_INFO, _("starting (version %s), pid %u user '%s'"), 
+  gconf_log (GCL_DEBUG, _("starting (version %s), pid %u user '%s'"), 
              VERSION, (guint)getpid(), g_get_user_name());
 
 #ifdef GCONF_ENABLE_DEBUG
@@ -685,6 +682,9 @@ main(int argc, char** argv)
   
   /* Session setup */
 #ifdef HAVE_SIGACTION
+  sigfillset (&full_mask);
+  sigprocmask (SIG_UNBLOCK, &full_mask, NULL);
+
   sigemptyset (&empty_mask);
   act.sa_handler = signal_handler;
   act.sa_mask    = empty_mask;
@@ -742,32 +742,9 @@ main(int argc, char** argv)
   gconf_set_daemon_ior (ior);
   CORBA_free (ior);
 
-  gconfd_dir = gconf_get_daemon_dir ();
-  lock_dir = gconf_get_lock_dir ();
-  
-  if (g_mkdir (gconfd_dir, 0700) < 0 && errno != EEXIST)
-    gconf_log (GCL_WARNING, _("Failed to create %s: %s"),
-               gconfd_dir, g_strerror (errno));
-  
-  if (!test_safe_tmp_dir (gconfd_dir))
-    {
-      err = g_error_new (GCONF_ERROR,
-                         GCONF_ERROR_LOCK_FAILED,
-                         _("Directory %s has a problem, gconfd can't use it"),
-                         gconfd_dir);
-      daemon_lock = NULL;
-    }
-  else
-    {
-      err = NULL;
-      
-      daemon_lock = gconf_get_lock (lock_dir, &err);
-    }
+  connection = get_on_d_bus ();
 
-  g_free (gconfd_dir);
-  g_free (lock_dir);
-
-  if (daemon_lock != NULL)
+  if (connection != NULL)
     {
       /* This loads backends and so on. It needs to be done before
        * we can handle any requests, so before we hit the
@@ -791,14 +768,8 @@ main(int argc, char** argv)
       close (write_byte_fd);
     }
   
-  if (daemon_lock == NULL)
+  if (connection == NULL)
     {
-      g_assert (err);
-
-      gconf_log (GCL_WARNING, _("Failed to get lock for daemon, exiting: %s"),
-                 err->message);
-      g_error_free (err);
-
       enter_shutdown ();
       shutdown_databases ();
       
@@ -841,14 +812,7 @@ main(int argc, char** argv)
 
   daemon_lock = NULL;
   
-  gconf_log (GCL_INFO, _("Exiting"));
-
-#ifdef HAVE_SYSLOG_H
-  closelog ();
-#endif
-
-  /* Can't do this due to stupid atexit() handler that calls g_log stuff */
-  /*   g_free (logname); */
+  gconf_log (GCL_DEBUG, _("Exiting"));
 
   return exit_code;
 }
@@ -920,12 +884,12 @@ gconf_main(void)
 
   if (main_loops == NULL)
     {
-      gulong timeout_len = 1000*60*0.5; /* 1 sec * 60 s/min * .5 min */
+      gulong timeout_len = 60*0.5; /* 60 s/min * .5 min */
       
       g_assert(timeout_id == 0);
-      timeout_id = g_timeout_add (timeout_len,
-                                  periodic_cleanup_timeout,
-                                  NULL);
+      timeout_id = g_timeout_add_seconds (timeout_len,
+                                          periodic_cleanup_timeout,
+                                          NULL);
 
     }
   
@@ -1354,7 +1318,7 @@ gconfd_check_in_shutdown (CORBA_Environment *ev)
       ConfigException* ce;
       
       ce = ConfigException__alloc();
-      ce->message = CORBA_string_dup("config server is currently shutting down");
+      ce->message = CORBA_string_dup("Configuration server is currently shutting down");
       ce->err_no = ConfigInShutdown;
 
       CORBA_exception_set(ev, CORBA_USER_EXCEPTION,
@@ -1457,14 +1421,14 @@ open_append_handle (GError **err)
 
 
       {
-        const gulong timeout_len = 1000*60*0.5; /* 1 sec * 60 s/min * 0.5 min */
+        const gulong timeout_len = 60*0.5; /* 60 s/min * 0.5 min */
 
         if (append_handle_timeout != 0)
           g_source_remove (append_handle_timeout);
         
-        append_handle_timeout = g_timeout_add (timeout_len,
-                                               close_append_handle_timeout,
-                                               NULL);
+        append_handle_timeout = g_timeout_add_seconds (timeout_len,
+                                                       close_append_handle_timeout,
+                                                       NULL);
       }
     }
 
@@ -1571,7 +1535,7 @@ logfile_save (void)
   fd = -1;
   
   /* Move the main saved state file aside, if it exists */
-  if (gconf_file_exists (logfile))
+  if (g_file_test(logfile, G_FILE_TEST_EXISTS))
     {
       tmpfile2 = g_strconcat (logfile, ".orig", NULL);
       if (g_rename (logfile, tmpfile2) < 0)
@@ -1587,7 +1551,7 @@ logfile_save (void)
   if (g_rename (tmpfile, logfile) < 0)
     {
       gconf_log (GCL_WARNING,
-                 _("Failed to move new save state file into place: %s"),
+                 _("Failed to move new saved state file into place: %s"),
                  g_strerror (errno));
 
       /* Try to restore old file */
@@ -1719,7 +1683,7 @@ parse_listener_entry (GHashTable *entries,
   if (err != NULL)
     {
       gconf_log (GCL_DEBUG,
-                 "Failed to unquote config source address from saved state file: %s",
+                 "Failed to unquote configuration source address from saved state file: %s",
                  err->message);
 
       g_error_free (err);
@@ -1938,7 +1902,7 @@ restore_client (const gchar *ior)
   
   if (ev._major != CORBA_NO_EXCEPTION)
     {
-      gconf_log (GCL_DEBUG, "Failed to update client in saved state file, probably the client no longer exists");
+      gconf_log (GCL_DEBUG, "Failed to update client in saved state file, the client probably no longer exists");
 
       goto finished;
     }
